from pathlib import Path

import click


def mkdir(base, path):
    if (base / path).exists():
        return f"Directory {path} exists; doing nothing"

    (base / path).mkdir(parents=True)
    return f"Directory {path} created"


def mkfile(base, path, contents, overwrite=False):
    file_exists = (base / path).exists()

    if file_exists and not overwrite:
        return f"File {path} exists; doing nothing"

    action = "overwritten" if overwrite else "created"

    with open(base / path, "w") as fd:
        fd.write(contents)

    return f"File {path} {action}"


def ensure_cwd_is_repo_root():
    """Exit if we're not in the root of the repo."""

    if not Path("./copilot").exists() or not Path("./copilot").is_dir():
        click.secho(
            "Cannot find copilot directory. Run this command in the root of the deployment repository.",
            bg="red",
        )
        exit(1)



import re

from schema import SchemaError


def validate_string(regex_pattern):
    def validator(string):
        if not re.match(regex_pattern, string):
            raise SchemaError(
                f"String '{string}' does not match the required pattern '{regex_pattern}'. For more details on valid string patterns see: https://aws.github.io/copilot-cli/docs/manifest/lb-web-service/"
            )
        return string

    return validator



from subprocess import run

import click


def get_lint_result(path: str, ignore_path: str = None, ignore_checks: str = None):
    command = ["cfn-lint", path]
    if ignore_path:
        command.extend(["--ignore-templates", ignore_path])
    if ignore_checks:
        command.extend(["--ignore-checks", ignore_checks])

    click.secho(f"\n>>> Running lint check", fg="yellow")
    click.secho(f"""    {" ".join(command)}\n""", fg="yellow")

    return run(command, capture_output=True)



#!/usr/bin/env python

import re
import subprocess

import boto3
import click

from dbt_copilot_helper.utils.click import ClickDocOptGroup
from dbt_copilot_helper.utils.versioning import (
    check_copilot_helper_version_needs_update,
)


@click.group(chain=True, cls=ClickDocOptGroup)
def svc():
    """AWS Copilot svc actions with DBT extras."""
    check_copilot_helper_version_needs_update()


@svc.command()
@click.option("--env", type=str, required=True)
@click.option("--name", type=str, required=True)
@click.option("--repository", type=str, required=True)
@click.option("--image-tag", type=str, required=False, show_default=True, default="latest")
def deploy(env, name, repository, image_tag):
    """Deploy image tag to a service, defaults to image tagged latest."""

    def get_all_tags_for_image(image_tag_needle):
        registry_id = boto3.client("sts").get_caller_identity()["Account"]
        ecr_client = boto3.client("ecr")
        try:
            response = ecr_client.describe_images(
                registryId=registry_id,
                repositoryName=repository,
                imageIds=[
                    {"imageTag": image_tag_needle},
                ],
            )
            return response["imageDetails"][0]["imageTags"]
        except ecr_client.exceptions.ImageNotFoundException:
            click.secho(
                f"""No image exists with the tag "{image_tag_needle}".""",
                fg="red",
            )
            exit(1)

    def get_commit_tag_for_latest_image(image_tags_haystack):
        try:
            filtered = [
                tag for tag in image_tags_haystack if re.match("(commit-[a-f0-9]{7,32})", tag)
            ]
            return filtered[0]
        except IndexError:
            click.secho(
                """The image tagged "latest" does not have a commit tag.""",
                fg="red",
            )
            exit(1)

    image_tags = get_all_tags_for_image(image_tag)

    if image_tag == "latest":
        image_tag = get_commit_tag_for_latest_image(image_tags)

    command = f"IMAGE_TAG={image_tag} copilot svc deploy --env {env} --name {name}"
    click.echo(f"Running: {command}")
    subprocess.call(
        command,
        shell=True,
    )
